from unv2x import *

if __name__=='__main__':
    helpmsg=""" UNV2ElmerUNV: Convert UNV file from Salome to ELMER UNV mesh
        usage: UNV2ElmerUNV unvfile prefix
        what it does: read unvfile, create an internal FEM object structure
                      in memory and writes the prefix.unv file
        As Salome does not write a color for finite elements, the different bodies
        cannot be distinguished. To define bodies, you need to create element groups
        called body1, body2 etc.. in Salome before you export the mesh.
        Moreover all elements in groups named X_* will just be deleted before conversion
        (usefull to delete all additionnal 1D & 2D elements created by Salome)
        Finally, a dataset 164 (units) will be added, with default units in SI (m,N)...
        Change the units in ELMER if your mesh is not in SI units.
    """
    if len(sys.argv)==3:
        unvfile=sys.argv[1]
        prefix=sys.argv[2]
        ls=os.linesep
        flag='    -1' + ls
        # read UNV file in FEM object structure
        UNV=UNVParser(unvfile)
        FEM=UNV.parse()
        # write ELMER UNV file
        # units
        fil=open(prefix + '.unv','w')
        fil.write(flag)
        fil.write('   164' + ls)
        fil.write('         1SI Units (m-N)      2' + ls)
        fil.write(('%25f%25f%25f' + ls + '%25f' + ls) % (1.000,1.000,1.000,273.000))
        fil.write(flag)
        # node         
        fil.write(flag)
        fil.write('  2411' + ls)
        for node in FEM.nodes:
            fil.write(('%10d%10d%10d%10d'+ls) % (node.id,0,0,0))
            fil.write((' %25f%25f%25f'+ls) % (node.coords[0],node.coords[1],node.coords[2]))
        fil.write(flag)            
        # elements
        #  first delete all elems from X_* groups
        #  & remove beam elements (id<33)
        idx2elem={}
        elem2idx={}
        count=0
        for elem in FEM.elems:
            idx2elem[count]=elem.id
            elem2idx[elem.id]=count
            count=count+1
        X_Ids=[]        
        for group in FEM.elemsets:
            if group.name.startswith('X_'):
                X_Ids.extend(group.items)
                FEM.elemsets.remove(group)                
        FEM.nelemsets=len(FEM.elemsets)        
        #  set colors corresponding to bodies
        bodylst=[]
        for group in FEM.elemsets:
            if group.name.startswith('body'):
                bodyId=int(group.name.strip()[4:])
                lst=group.items
                bodylst.append(lst)
        for i in range(len(bodylst)):
            for elem in bodylst[i]:
                 idx=elem2idx[elem]
                 FEM.elems[idx].color=i+1
                 FEM.elems[idx].material=i+1                   
##        for elem in FEM.elems:
##            for i in range(len(bodylst)):
##                if elem.id in bodylst[i]:
##                    idx=FEM.elems.index(elem)
##                    FEM.elems[idx].color=i+1
##                    FEM.elems[idx].material=i+1
        # write dataset 2412
        fil.write(flag)
        fil.write('  2412' + ls)
        for elem in FEM.elems:
            if not(elem.id in X_Ids):
                if elem.type>32:
                    nnodes=elem.nnodes
                    if elem.type==11:
                        print 'problem...',elem.type
                    dat=[elem.id,elem.type,elem.material,elem.material,elem.color,elem.nnodes]            
                    fil.write((('%10d' * 6) + ls) % tuple(dat))
                    count=0
                    lst=elem.cntvt
                    for i in range(nnodes):
                        count=count+1
                        if (count<8)&(i<>nnodes-1):
                            fil.write('%10d' % lst.pop(0))
                        else:
                            fil.write(('%10d'+ls) % lst.pop(0))
                            count=0
        fil.write(flag)
        # group definitions
        fil.write(flag)
        fil.write('  2417' + ls)
        for group in (FEM.nodesets + FEM.elemsets):            
            fil.write((('%10d' * 7)+ls) % (group.id,0,0,0,0,0,group.nitems))
            fil.write('%20s' % (group.name))
            count=0            
            lst=group.items                        
            for i in range(group.nitems):
                count=count+1
                if (count<4)&(i<>group.nitems-1):
                    fil.write('%10d%10d' % (group.type,lst.pop(0)))
                else:
                    fil.write(('%10d%10d'+ls) % (group.type,lst.pop(0)))
                    count=0
        fil.write(flag)
        fil.close()
        
    else:
        print(helpmsg)
